/*
 * IHM permettant de communiquer avec l'Arduino pour le projet "Vertical2024"
 */
using System.IO.Ports;

namespace Vertical2024 {
    public partial class Form1 : Form {
        SerialPort serialPort;
        bool bMoteurChanged = false;
        bool bKUChanged = false;
        bool bK0Changed = false;
        bool bK1Changed = false;
        bool bK2Changed = false;
        bool bK3Changed = false;
        bool bAngleZeroChanged = false;
        public Form1() {
            InitializeComponent();

            // Valeurs par dfaut des trackbars
            initTrackbarsK();

            // Paramtrage du port srie
            serialPort = new SerialPort();
            serialPort.PortName = "COM7";
            serialPort.BaudRate = 38400;
            serialPort.Open();

            // Lecture en continu du port srie, et criture du contenu dans textBoxLog
            Task.Run(() => {
                while (serialPort.IsOpen) {
                    try {
                        string message = serialPort.ReadLine();
                        message = message.Replace("#", "\r\n");
                        message = "---------------------------------------------\r\nDonnes renvoyes par l'Arduino Uno\r\n---------------------------------------------\r\n\r\n" + message;
                        textBoxLog.Invoke(new MethodInvoker(delegate { textBoxLog.Text = message; })); // Invoke ncessaire car nous ne pas sommes ici dans le thread du GUI
                    }
                    catch (Exception e) {
                        MessageBox.Show(e.Message);
                    }
                }
            });
            // Timer destin  l'criture sur le port srie (pour ne pas crire trop vite dessus, lorsque les trackbars sont utilises)
            timer1.Interval = 50;
            timer1.Start();
        }
        private void initTrackbarsK() {
            trackBarKU.Value = 176;
            trackBarK0.Value = -245;
            trackBarK1.Value = -220;
            trackBarK2.Value = 1310;
            trackBarK3.Value = 91;
        }
        private void trackBarKU_Scroll(object sender, EventArgs e) {
            bKUChanged = true;
        }
        private void trackBarK0_Scroll(object sender, EventArgs e) {
            bK0Changed = true;
        }
        private void trackBarK1_Scroll(object sender, EventArgs e) {
            bK1Changed = true;
        }
        private void trackBarK2_Scroll(object sender, EventArgs e) {
            bK2Changed = true;
        }
        private void trackBarK3_Scroll(object sender, EventArgs e) {
            bK3Changed = true;
        }
        private void btnAngleZero_Click(object sender, EventArgs e) {
            bAngleZeroChanged = true;
        }
        private void trackBarMoteur_Scroll(object sender, EventArgs e) {
            bMoteurChanged = true;
        }
        private void btnAsservissement_Click(object sender, EventArgs e) {
            serialPort.Write("a");  // asservissement
        }
        private void timer1_Tick(object sender, EventArgs e) {
            if (bKUChanged) {
                serialPort.Write(trackBarKU.Value + "u"); // exemple : 185u
                bKUChanged = false;
            }
            if (bK0Changed) {
                serialPort.Write(trackBarK0.Value + "A"); // exemple : 203A
                bK0Changed = false;
            }
            if (bK1Changed) {
                serialPort.Write(trackBarK1.Value + "B");
                bK1Changed = false;
            }
            if (bK2Changed) {
                serialPort.Write(trackBarK2.Value + "C");
                bK2Changed = false;
            }
            if (bK3Changed) {
                serialPort.Write(trackBarK3.Value + "D");
                bK3Changed = false;
            }
            if (bMoteurChanged) {
                int pMoteur = -trackBarMoteur.Value;
                serialPort.Write(pMoteur + "m");
                bMoteurChanged = false;
            }
            if (bAngleZeroChanged) {
                serialPort.Write("z");
                bAngleZeroChanged = false;
            }
        }
        private void btnMoteur0_Click(object sender, EventArgs e) {
            trackBarMoteur.Value = 0;
            bMoteurChanged = true;
        }
        private void btnResetK_Click(object sender, EventArgs e) {
            initTrackbarsK();
            bK0Changed = true;
            bK1Changed = true;
            bK2Changed = true;
            bK3Changed = true;
            bKUChanged = true;
        }
    }
}